@@ -7,7 +7,6 @@ import cro_validate.classes.parameter_classes as Parameters
 import cro_validate.classes.schema_classes as Schemas
 import cro_validate.classes.name_strategy_classes as NameStrategies
 import cro_validate.classes.util_classes as Utils
-import cro_validate.classes.dependency_classes as Dependencies
 
 
 class Meta:
@@ -23,7 +22,7 @@ class DefaultDefinitionMeta(Meta):
 	def __init__(self, component_name_strategy=Utils.Empty):
 		if not Utils.Empty.isempty(component_name_strategy):
 			self.component_name_strategy = component_name_strategy
-		
+
 
 	def initialize(self, definition, component_name_suffix='Model', display_name=None):
 		if definition.is_object():
@@ -48,7 +47,6 @@ class Definition:
 	internal = False
 	rules = []
 	meta = DefaultDefinitionMeta()
-	dependency_resolver = Dependencies.DefaultDefinitionDependencyResolver()
 
 	def to_json_dict(self):
 		def _is_default(name, value):
@@ -59,8 +57,7 @@ class Definition:
 		def _set(target, name, value):
 			if _is_default(name, value):
 				return
-			target[name] = value		
-		dependency_resolver_type = Utils.ClassName.class_fqn(self.dependency_resolver)
+			target[name] = value
 		fmt = None
 		if self.data_format is not None:
 			if isinstance(self.data_format, Enum):
@@ -111,8 +108,7 @@ class Definition:
 				deprecated,
 				internal,
 				rules,
-				meta,
-				dependency_resolver
+				meta
 			):
 		def _set_member(name, value):
 			if Utils.Empty.isempty(value):
@@ -128,10 +124,8 @@ class Definition:
 		_set_member('nullable', nullable)
 		_set_member('deprecated', deprecated)
 		_set_member('internal', internal)
-		_set_member('dependencies', set())
 		_set_member('rules', rules)
 		_set_member('meta', meta)
-		_set_member('dependency_resolver', dependency_resolver)
 		# Name
 		######
 		self.name = Config.definition_name_strategy.create_name(self, self.name)
@@ -158,18 +152,15 @@ class Definition:
 			format_definition = Index.get(self.data_format)
 			self.data_format = format_definition.data_format
 		elif self.data_type is DataType.OneOf:
-			self.data_format = self.dependency_resolver.list_dependent_definition_names(self.name)
+			pass
 		# Validator
 		###########
 		if self.is_object():
 			self.validator = self._get_obj_validator()
 		elif self.is_array():
-			self.validator = self._validate_array	
+			self.validator = self._validate_array
 		else:
 			self.validator = self._assign_value
-		# Dependencies
-		##############
-		self.dependencies = set(self.dependency_resolver.list_dependency_fields(self.name))
 		# Examples
 		##########
 		if not self.examples:
@@ -181,73 +172,9 @@ class Definition:
 		######
 		self.meta.initialize(self)
 
-	def _get_obj_model_validator(self):
-		display_name = self.data_format.model_name
-		if self.data_format.display_name is not None:
-			display_name = str(self.data_format.display_name)
-		validator = Schemas.ModelValidator(
-				name=display_name,
-				allow_unknown_fields=self.data_format.allow_unknown_fields,
-				case_sensitive=self.data_format.case_sensitive,
-				load_defaults=self.data_format.load_defaults
-			)
-		required = set()
-		optional = set()
-		ignored = set()
-		unvalidated = set()
-		definition_names = {}
-		input_names = {}
-		output_names = {}
-		dependencies = {}
-		default_values = {}
-		fields = self.list_fields()
-		for name in fields:
-			field = fields[name]
-			field_definition_name = name
-			if field.definition_name is not None:
-				field_definition_name = field.definition_name
-			field_definition = Index.get(field_definition_name)
-			if field.unvalidated is not True:
-				dependencies[name] = field_definition.dependencies
-			if field is None:
-				field = Schemas.Field()
-			if field.required:
-				required.add(name)
-			else:
-				optional.add(name)
-			if field.ignored:
-				ignored.add(name)
-			if field.unvalidated:
-				unvalidated.add(name)
-			if field.definition_name:
-				definition_names[name] = field.definition_name
-			if field.output_name:
-				output_names[name] = field.output_name
-			if field.input_name:
-				input_names[name] = field.input_name
-			if not Utils.Empty.isempty(field.default_value):
-				default_values[name] = field.default_value
-			else:
-				default_values[name] = field_definition.default_value
-		validator.add_spec(
-				required=required,
-				optional=optional,
-				ignored=ignored,
-				unvalidated=unvalidated,
-				definition_names=definition_names,
-				input_names=input_names,
-				output_names=output_names,
-				dependencies=dependencies,
-				default_values=default_values
-			)
-		return validator
-
 	def _get_obj_validator(self):
-		model_validator = self._get_obj_model_validator()
-		validator = Schemas.Validator(
-				self.name,
-				model_validator
-			)
+		model_validator = Schemas.ModelValidator(self.data_format)
+		validator = Schemas.Validator(self.name, model_validator)
 		return validator
 
 	def _validate_array(
@@ -257,7 +184,6 @@ class Definition:
 				field_name,
 				definition,
 				value,
-				dependent_values,
 				**rules_kw):
 		if isinstance(value, list):
 			pass
@@ -268,7 +194,7 @@ class Definition:
 		items = []
 		i = 0
 		for entry in value:
-			item = Index.validate_input(None, field_fqn + '[' + str(i) + ']', field_name, self.data_format, entry, dependent_values)
+			item = Index.validate_input(None, field_fqn + '[' + str(i) + ']', field_name, self.data_format, entry)
 			items.append(item[field_name])
 			i = i + 1
 		results[field_name] = items
@@ -280,7 +206,6 @@ class Definition:
 				field_name,
 				definition,
 				value,
-				dependent_values,
 				**rules_kw):
 		results[field_name] = value
 
@@ -291,7 +216,6 @@ class Definition:
 				field_name,
 				definition,
 				value,
-				dependent_values,
 				**rules_kw
 			):
 		'''
@@ -307,10 +231,6 @@ class Definition:
 				field_fqn = field_name
 				if self.data_type == DataType.Object:
 					field_fqn = self.validator.model_validator.name
-			dependent_definition_name = self.dependency_resolver.get_dependent_definition(
-					field_fqn,
-					dependent_values
-				)
 			if value is None:
 				if self.nullable is True:
 					results[field_name] = None
@@ -332,26 +252,14 @@ class Definition:
 					self,
 					**rules_kw
 				)
-			if dependent_definition_name is not None:
-				Index.validate_input(
-						results,
-						field_fqn,
-						field_name,
-						dependent_definition_name,
-						results[field_name],
-						dependent_values,
-						**rules_kw
-					)
-			else:
-				self.validator(
-						results,
-						field_fqn,
-						field_name,
-						self,
-						results[field_name],
-						dependent_values,
-						**rules_kw
-					)
+			self.validator(
+					results,
+					field_fqn,
+					field_name,
+					self,
+					results[field_name],
+					**rules_kw
+				)
 			for rule in self.rules:
 				results[field_name] = rule.execute(field_fqn, results[field_name], **rules_kw)
 			return results
@@ -400,6 +308,11 @@ class Definition:
 			return True
 		return False
 
+	def is_one_of(self):
+		if self.data_type == DataType.OneOf:
+			return True
+		return False
+
 	def is_primitive(self):
 		if self.is_object() or self.is_array():
 			return False
@@ -473,30 +386,65 @@ class DefinitionJsonDeserializer:
 		if not Index.exists(kw['definition_name']):
 			dependent_definition_profile = self.deserialize(root, kw['definition_name'])
 			Index.register_definition(**dependent_definition_profile)
-		self._set(obj, kw, 'dependencies')
 		self._set(obj, kw, 'ignored')
 		self._set(obj, kw, 'input_name')
 		self._set(obj, kw, 'output_name')
 		self._set(obj, kw, 'required')
 		self._set(obj, kw, 'unvalidated')
-		if 'dependencies' in kw and isinstance(kw['dependencies'], list):
-			kw['dependencies'] = set(kw['dependencies'])
 		field = Schemas.Field(**kw)
 		return field
-		
+
 	def _deserialize_schema(self, root, obj, name):
 		kw = {}
 		self._set(obj, kw, 'allow_unknown_fields')
 		self._set(obj, kw, 'case_sensitive')
 		self._set(obj, kw, 'display_name')
+		self._set(obj, kw, 'load_defaults')
 		self._set(obj, kw, 'model')
+		self._set(obj, kw, 'dependency_resolver')
 		kw['model_name'] = name
 		model = {}
+		field_init_actions = []
+		if 'field_init_actions' in obj:
+			for entry in obj['field_init_actions']:
+				init_action_kw = {}
+				if 'kw' in entry:
+					init_action_kw = entry['kw']
+				init_action_class_name = entry['type']
+				init_action = self._instantiate(init_action_class_name, **init_action_kw)
+				field_init_actions.append(init_action)
+		if 'dependency_resolver' in obj:
+			resolver_class_name = obj['dependency_resolver']['type']
+			resolver_kw = {}
+			if 'kw' in obj['dependency_resolver']:
+				resolver_kw = obj['dependency_resolver']['kw']
+			resolver = self._instantiate(resolver_class_name, **resolver_kw)
+			kw['dependency_resolver'] = resolver
+		if 'inherits' in obj:
+			for inherited_definition_name in obj['inherits']:
+				if not Index.exists(inherited_definition_name):
+					dependent_definition_profile = self.deserialize(root, inherited_definition_name)
+					Index.register_definition(**dependent_definition_profile)
+				parent_definition = Index.get(inherited_definition_name)
+				inherited_fields = parent_definition.list_fields()
+				for field_name in inherited_fields:
+					model[field_name] = inherited_fields[field_name]
 		if 'model' in obj:
-			for name in obj['model']:
-				field = self._deserialize_schema_field(root, name, obj['model'][name])
-				model[name] = field
-		kw['model'] = model
+			for field_name in obj['model']:
+				field = self._deserialize_schema_field(root, field_name, obj['model'][field_name])
+				model[field_name] = field
+		initialized_model = {}
+		for action in field_init_actions:
+			action.validate_schema_model(model)
+		for field_name in model:
+			field = model[field_name]
+			for action in field_init_actions:
+				field = action(field_name, field)
+				if field is None:
+					break
+			if field is not None:
+				initialized_model[field_name] = field
+		kw['model'] = initialized_model
 		schema = Schemas.Schema(**kw)
 		return schema
 
@@ -529,7 +477,6 @@ class DefinitionJsonDeserializer:
 			'data_format': Utils.Empty,
 			'default_value': Utils.Empty,
 			'meta': Utils.Empty,
-			'dependency_resolver': Utils.Empty,
 			'rules': Utils.Empty
 		}
 		namespace = {k:k for k in root}
@@ -633,8 +580,7 @@ class Index:
 				deprecated,
 				internal,
 				rules,
-				meta,
-				dependency_resolver
+				meta
 			):
 		definition = Definition(
 				name=name,
@@ -648,8 +594,7 @@ class Index:
 				deprecated=deprecated,
 				internal=internal,
 				rules=rules,
-				meta=meta,
-				dependency_resolver=dependency_resolver
+				meta=meta
 			)
 		definition_name = definition.get_name()
 		names = set()
@@ -718,12 +663,11 @@ class Index:
 				field_name,
 				definition_or_name,
 				value,
-				dependent_values={},
 				**rules_kw
 			):
 		definition = Index.get(definition_or_name)
 		results = Parameters.Index.ensure(validated)
-		definition.validate(results, field_fqn, field_name, definition, value, dependent_values, **rules_kw)
+		definition.validate(results, field_fqn, field_name, definition, value, **rules_kw)
 		return results
 
 	def ensure_alias(name, alias):
@@ -736,15 +680,6 @@ class Index:
 		result.sort()
 		return result
 
-	def list_dependent_definitions(definition_name):
-		results = set()
-		definition = Index.get(definition_name)
-		if definition.data_type == DataType.Object:
-			result = definition.validator.list_definition_names()
-		elif definition.data_type == DataType.Array:
-			results = Index.list_dependent_definitions(definition.data_format)
-		return results
-
 	def list_fields(name):
 		definition = Index.get(name)
 		if definition.data_type == DataType.Object:
@@ -752,4 +687,4 @@ class Index:
 		return [name]
 
 	def clear():
-		Index._idx = {}
\ No newline at end of file
+		Index._idx = {}
