@@ -11,6 +11,7 @@ class Schema:
 	display_name = None
 	case_sensitive = True
 	load_defaults = False
+	dependency_resolver = None
 
 	def __init__(
 				self,
@@ -19,7 +20,8 @@ class Schema:
 				allow_unknown_fields=Utils.Empty,
 				display_name=Utils.Empty,
 				case_sensitive=Utils.Empty,
-				load_defaults=Utils.Empty
+				load_defaults=Utils.Empty,
+				dependency_resolver=Utils.Empty
 			):
 		def _set_member(name, value):
 			if Utils.Empty.isempty(value):
@@ -35,6 +37,7 @@ class Schema:
 		_set_member('display_name', display_name)
 		_set_member('case_sensitive', case_sensitive)
 		_set_member('load_defaults', load_defaults)
+		_set_member('dependency_resolver', dependency_resolver)
 		if self.model_name is None:
 			self.model_name = model.__class__.__name__
 
@@ -84,7 +87,7 @@ class Field:
 	required = True
 	ignored = False
 	unvalidated = False
-	dependencies = set()
+	dependency_resolver = None
 	default_value = Utils.Empty
 
 	def __init__(
@@ -95,7 +98,7 @@ class Field:
 				required=Utils.Empty,
 				ignored=Utils.Empty,
 				unvalidated=Utils.Empty,
-				dependencies=Utils.Empty,
+				dependency_resolver=Utils.Empty,
 				default_value=Utils.Empty
 			):
 		def _set_member(name, value):
@@ -108,9 +111,10 @@ class Field:
 		_set_member('required', required)
 		_set_member('ignored', ignored)
 		_set_member('unvalidated', unvalidated)
-		_set_member('dependencies', dependencies)
+		_set_member('dependency_resolver', dependency_resolver)
 		_set_member('default_value', default_value)
 
+	# Broken
 	def to_json_dict(self, field_name):
 		def _is_default(name, value):
 			if getattr(self.__class__, name) == value:
@@ -134,7 +138,7 @@ class Field:
 		_set(result, 'required', self.required)
 		_set(result, 'ignored', self.ignored)
 		_set(result, 'unvalidated', self.unvalidated)
-		_set(result, 'dependencies', self.dependencies)
+		_set(result, 'dependency_resolver', self.dependency_resolver)
 		_set(result, 'default_value', self.default_value)
 		if 'default_value' in result:
 			default_value_type = Utils.ClassName.class_fqn(self.default_value)
@@ -143,7 +147,7 @@ class Field:
 
 
 class Validator:
-	def _validate(self, validated, fqn, field_name, definition, value, dependent_values, **rules_kw):
+	def _validate(self, validated, fqn, field_name, definition, value, **rules_kw):
 		results = Parameters.Index.ensure(validated)
 		cb = Definitions.Index.validate_input
 		model_result = self.model_validator(None, fqn, field_name, definition, value, cb, **rules_kw)
@@ -153,8 +157,8 @@ class Validator:
 		self.name = name
 		self.model_validator = model_validator
 
-	def __call__(self, results, fqn, field_name, definition, value, dependent_values, **rules_kw):
-		return self._validate(results, fqn, field_name, definition, value, dependent_values, **rules_kw)
+	def __call__(self, results, fqn, field_name, definition, value, **rules_kw):
+		return self._validate(results, fqn, field_name, definition, value, **rules_kw)
 
 	def is_field_required(self, field_name):
 		if field_name in self.model_validator.required:
@@ -170,14 +174,11 @@ class Validator:
 	def get_field_input_display_name(self, field_name):
 		return self.model_validator.get_field_input_display_name(field_name)
 
-	def get_dependency_order(self):
-		return self.model_validator.get_dependency_order()
-
-	def list_field_dependencies(self, field_name):
-		return self.model_validator.dependencies[field_name]
-
 	def list_field_names(self):
-		return self.get_dependency_order()
+		field_names = set()
+		field_names.update(self.model_validator.required)
+		field_names.update(self.model_validator.optional)
+		return field_names
 
 	def list_definition_names(self):
 		field_names = set()
@@ -210,11 +211,8 @@ class Validator:
 
 
 class ModelValidator:
-	def __init__(self, name, allow_unknown_fields, case_sensitive, load_defaults):
-		self.name = name
-		self.allow_unknown_fields = allow_unknown_fields
-		self.case_sensitive = case_sensitive
-		self.load_defaults = load_defaults
+	def __init__(self, schema):
+		self.schema = schema
 		self.required = set()
 		self.optional = set()
 		self.ignored = set()
@@ -225,23 +223,154 @@ class ModelValidator:
 		self.default_values = {}
 		self.input_field_display_names = {}
 		self.output_names = {}
-		self.dependencies = {}
-		self.dependency_order = None
+		self.dependency_order = []
+		self.load_schema()
+		self.name = schema.model_name
+		if schema.display_name is not None:
+			self.name = schema.display_name
+
+	def add_required(self, names):
+		self.required.update(names)
+
+	def add_optional(self, names):
+		self.optional.update(names)
+
+	def add_ignored(self, names):
+		self.ignored.update(names)
+
+	def add_unvalidated(self, names):
+		self.unvalidated.update(names)
+
+	def add_definition_names(self, definition_names):
+		self.definition_names.update(definition_names)
+
+	def add_output_names(self, output_names):
+		self.output_names.update(output_names)
+
+	def add_input_names(self, input_names):
+		normalized = input_names
+		if not self.schema.case_sensitive:
+			normalized = {k.lower():input_names[k].lower() for k in input_names}
+		self.input_field_display_names.update(input_names)
+		self.input_field_names.update(normalized)
+		self.input_value_names.update({normalized[k]:k for k in normalized})
+
+	def add_default_values(self, default_values):
+		for field_name in default_values:
+			value = default_values[field_name]
+			if Utils.Empty.isempty(value):
+				continue
+			self.default_values[field_name] = value
+
+	def update_dependency_order(self):
+		dependent_fields = set()
+		independent_fields = set()
+		for entry in self.definition_names:
+			definition_name = self.definition_names[entry]
+			definition = Definitions.Index.get(definition_name)
+			if definition.is_one_of():
+				dependent_fields.add(entry)
+			else:
+				independent_fields.add(entry)
+		self.dependency_order = []
+		self.dependency_order.extend(independent_fields)
+		self.dependency_order.extend(dependent_fields)
+
+	def add_spec(self,
+				required,
+				optional,
+				ignored,
+				unvalidated,
+				definition_names={},
+				input_names={},
+				output_names={},
+				default_values={}
+			):
+		if self.schema.case_sensitive:
+			required = {k.lower() for k in required}
+			optional = {k.lower() for k in optional}
+			ignored = {k.lower() for k in ignored}
+			unvalidated = {k.lower() for k in unvalidated}
+			ignored = {k.lower() for k in ignored}
+			default_values = {k.lower():default_values[k] for k in default_values}
+		self.add_required(required)
+		self.add_optional(optional)
+		self.add_ignored(ignored)
+		self.add_unvalidated(unvalidated)
+		self.add_definition_names(definition_names)
+		self.add_input_names(input_names)
+		self.add_output_names(output_names)
+		self.add_default_values(default_values)
+		self.update_dependency_order()
+
+	def load_schema(self):
+		required = set()
+		optional = set()
+		ignored = set()
+		unvalidated = set()
+		definition_names = {}
+		input_names = {}
+		output_names = {}
+		default_values = {}
+		fields = {}
+		if isinstance(self.schema.model, dict):
+			fields = self.schema.model
+		else:
+			for entry in dir(self.schema.model):
+				if entry.startswith('__'):
+					continue
+				field = getattr(self.schema.model, entry)
+				if field is not None and isinstance(field, Field) is not True:
+					continue
+				fields[entry] = field
+		for name in fields:
+			field = fields[name]
+			if field is None:
+				field = Field()
+			field_definition_name = name
+			if field.definition_name is not None:
+				field_definition_name = field.definition_name
+			field_definition = Definitions.Index.get(field_definition_name)
+			if field.required:
+				required.add(name)
+			else:
+				optional.add(name)
+			if field.ignored:
+				ignored.add(name)
+			if field.unvalidated:
+				unvalidated.add(name)
+			if field.definition_name:
+				definition_names[name] = field.definition_name
+			if field.output_name:
+				output_names[name] = field.output_name
+			if field.input_name:
+				input_names[name] = field.input_name
+			if not Utils.Empty.isempty(field.default_value):
+				default_values[name] = field.default_value
+			else:
+				default_values[name] = field_definition.default_value
+		self.add_spec(
+				required=required,
+				optional=optional,
+				ignored=ignored,
+				unvalidated=unvalidated,
+				definition_names=definition_names,
+				input_names=input_names,
+				output_names=output_names,
+				default_values=default_values
+			)
 
 	def get_field_definition_name(self, field_name):
+		definition_name = field_name
 		if field_name in self.definition_names:
-			return self.definition_names[field_name]
-		return field_name
+			definition_name = self.definition_names[field_name]
+		return definition_name
 
 	def get_field_input_name(self, field_name):
 		if field_name in self.input_field_names:
 			return self.input_field_names[field_name]
 		return field_name
 
-	def get_dependency_order(self):
-		self._update_dependency_order()
-		return self.dependency_order
-
 	def get_field_input_display_name(self, field_name):
 		if field_name in self.input_field_names:
 			return self.input_field_display_names[field_name]
@@ -251,7 +380,7 @@ class ModelValidator:
 		result = []
 		if isinstance(vector, dict):
 			for k in vector:
-				if self.case_sensitive:
+				if self.schema.case_sensitive:
 					k = k.lower()
 				if k in self.input_value_names:
 					result.append(self.input_value_names[k])
@@ -266,7 +395,7 @@ class ModelValidator:
 						continue
 					if callable(getattr(vector, attribute_name)):
 						continue
-					if self.case_sensitive:
+					if self.schema.case_sensitive:
 						attribute_name = attribute_name.lower()
 					if attribute_name in self.input_value_names:
 						result.append(self.input_value_names[attribute_name])
@@ -304,6 +433,22 @@ class ModelValidator:
 				setattr(target, value_name, value)
 		return value
 
+	def _resolve_field_definition(self, fqn, field_name, dependency_values):
+		definition_name = self.get_field_definition_name(field_name)
+		definition = Definitions.Index.get(definition_name)
+		if definition.is_one_of():
+			if self.schema.dependency_resolver is None:
+				raise Config.exception_factory.create_input_error(fqn, 'Schema field dependency resolver missing.')
+			try:
+				definition_name = self.schema.dependency_resolver.resolve(fqn, field_name, dependency_values)
+				if definition_name not in definition.get_data_format():
+					msg = "Schema field cannot be '{0}' (must be one of: {1})".format(definition_name, definition.get_data_format())
+					raise Config.exception_factory.create_internal_error(fqn, msg)
+				definition = Definitions.Index.get(definition_name)
+			except Exception as ex:
+				raise Config.exception_factory.create_input_error(fqn, 'Schema field resolution failed: {0}.'.format(ex), exception=ex)
+		return definition
+
 	def _rename_output(self, src_field_name, target_field_name, target):
 		if isinstance(target, dict):
 			tmp = target[src_field_name]
@@ -328,17 +473,16 @@ class ModelValidator:
 		field_names = self._list_input_field_names(normalized)
 		kw = Parameters.Index()
 		missing = set(self.required)
-		self._update_dependency_order()
 		# Build kw
 		##########
-		if self.load_defaults is True:
+		if self.schema.load_defaults is True:
 			for field_name in self.optional:
 				kw[field_name] = self.default_values[field_name]
 		for field_name in field_names:
 			if field_name in missing:
 				missing.remove(field_name)
 			elif field_name not in self.optional:
-				if not self.allow_unknown_fields:
+				if not self.schema.allow_unknown_fields:
 					display_name = self.get_field_input_display_name(field_name)
 					raise Config.exception_factory.create_input_error(Config.definition_name_strategy.get_fqn(fqn, display_name), 'Unknown field.')
 				else:
@@ -352,33 +496,29 @@ class ModelValidator:
 			for entry in missing:
 				display_names.append(self.get_field_input_display_name(entry))
 			raise Config.exception_factory.create_input_error(fqn, 'Missing required values: {0}'.format(', '.join(display_names)))
-		for entry in self.dependency_order:
+		for field_name in self.dependency_order:
+			display_name = self.get_field_input_display_name(field_name)
+			child_fqn = Config.definition_name_strategy.get_fqn(fqn, display_name)
 			# Unvalidated
 			#############
-			if entry in self.unvalidated:
-				result[entry] = self._get_input_value(fqn, entry, normalized)
+			if field_name in self.unvalidated:
+				result[field_name] = self._get_input_value(fqn, field_name, normalized)
 				continue
 			# Default Value
 			###############
-			field_definition_name = self.get_field_definition_name(entry)
-			field_definition = Definitions.Index.get(field_definition_name)
-			if entry not in kw:
-				if field_definition.has_default_value() and entry in self.required:
-					result[entry] = field_definition.get_default_value(entry)
+			field_definition = self._resolve_field_definition(child_fqn, field_name, result)
+			if field_name not in kw:
+				if field_definition.has_default_value() and field_name in self.required:
+					result[field_name] = field_definition.get_default_value(field_name)
 				continue
 			# Normal Field
 			##############
-			dependencies = field_definition.dependencies
-			dependent_values = {k:kw[k] for k in kw if k in dependencies}
-			display_name = self.get_field_input_display_name(entry)
-			child_fqn = Config.definition_name_strategy.get_fqn(fqn, display_name)
 			definition_result = callback(
 					validated=None,
 					field_fqn=child_fqn,
-					field_name=entry,
-					definition_or_name=field_definition_name,
-					value=kw[entry],
-					dependent_values=dependent_values,
+					field_name=field_name,
+					definition_or_name=field_definition,
+					value=kw[field_name],
 					**rules_kw)
 			for definition_result_entry in definition_result:
 				self._set_output_value(
@@ -396,128 +536,9 @@ class ModelValidator:
 			if entry in self.output_names:
 				output_name = self.output_names[entry]
 				self._rename_output(entry, output_name, result)
-		
+
 		return result
 
-	def _update_dependency_order(self):
-		'''
-		Depth-first topo sort
-		'''
-		# Initialize
-		############
-		if self.dependency_order is not None:
-			return
-		dependencies_idx = {}
-		for k in self.required:
-			if k in self.unvalidated:
-				continue
-			definition_name = self.get_field_definition_name(k)
-			dependencies_idx[k] = Definitions.Index.get(definition_name).dependencies
-		for k in self.optional:
-			if k in self.unvalidated:
-				continue
-			definition_name = self.get_field_definition_name(k)
-			dependencies_idx[k] = Definitions.Index.get(definition_name).dependencies
-		remaining = set([k for k in dependencies_idx.keys()])
-		result = []
-		# Visit
-		#######
-		def visit(k, branch):
-			if k not in remaining:
-				return
-			if k in branch:
-				raise Config.exception_factory.create_internal_error(self.name, 'Dependency circular reference.')
-			branch.add(k)
-			for edge in dependencies_idx[k]:
-				visit(edge, branch)
-			branch.remove(k)
-			remaining.remove(k)
-			result.append(k)
-		# Iterate
-		#########
-		while len(remaining) > 0:
-			branch = set()
-			for k in remaining:
-				selected = k
-				break
-			visit(selected, branch)
-		self.dependency_order = result
-
-	def add_required(self, names):
-		for name in names:
-			if name not in self.dependencies:
-				self.dependencies[name] = set()
-		self.required.update(names)
-
-	def add_optional(self, names):
-		for name in names:
-			if name not in self.dependencies:
-				self.dependencies[name] = set()
-		self.optional.update(names)
-
-	def add_ignored(self, names):
-		self.ignored.update(names)
-
-	def add_unvalidated(self, names):
-		self.unvalidated.update(names)
-
-	def add_definition_names(self, definition_names):
-		self.definition_names.update(definition_names)
-
-	def add_output_names(self, output_names):
-		self.output_names.update(output_names)
-
-	def add_input_names(self, input_names):
-		normalized = input_names
-		if not self.case_sensitive:
-			normalized = {k.lower():input_names[k].lower() for k in input_names}
-		self.input_field_display_names.update(input_names)
-		self.input_field_names.update(normalized)
-		self.input_value_names.update({normalized[k]:k for k in normalized})
-
-	def add_dependencies(self, new_dependencies):
-		for name in new_dependencies:
-			if name not in self.dependencies:
-				raise Config.exception_factory.create_internal_error(str(add_dependencies) + '.' + str(name), 'Unknown field')
-			self.dependencies[name].update(new_dependencies[name])
-		self.dependency_order = None
-
-	def add_default_values(self, default_values):
-		for field_name in default_values:
-			value = default_values[field_name]
-			if Utils.Empty.isempty(value):
-				continue
-			self.default_values[field_name] = value
-
-	def add_spec(self,
-				required,
-				optional,
-				ignored,
-				unvalidated,
-				definition_names={},
-				input_names={},
-				output_names={},
-				dependencies={},
-				default_values={}
-			):
-		if self.case_sensitive:
-			required = {k.lower() for k in required}
-			optional = {k.lower() for k in optional}
-			ignored = {k.lower() for k in ignored}
-			unvalidated = {k.lower() for k in unvalidated}
-			ignored = {k.lower() for k in ignored}
-			dependencies = {k.lower():{k1.lower() for k1 in dependencies[k]} for k in dependencies}
-			default_values = {k.lower():default_values[k] for k in default_values}
-		self.add_required(required)
-		self.add_optional(optional)
-		self.add_ignored(ignored)
-		self.add_unvalidated(unvalidated)
-		self.add_definition_names(definition_names)
-		self.add_input_names(input_names)
-		self.add_output_names(output_names)
-		self.add_dependencies(dependencies)
-		self.add_default_values(default_values)
-
 	def __call__(self, validated, fqn, field_name, definition, values, cb, **rules_kw):
 		return self._iterate(
 				validated,
@@ -525,4 +546,4 @@ class ModelValidator:
 				values,
 				cb,
 				**rules_kw
-			)
\ No newline at end of file
+			)
